r"""
Dense univariate polynomials over `\RR`, implemented using MPFR

TESTS:

Check that operations with numpy elements work well (see :issue:`18076` and
:issue:`8426`)::

    sage: # needs numpy
    sage: import numpy
    sage: x = polygen(RR)
    sage: x * numpy.int32('1')
    x
    sage: numpy.int32('1') * x
    x
    sage: x * numpy.int64('1')
    x
    sage: numpy.int64('1') * x
    x
    sage: x * numpy.float32('1.5')
    1.50000000000000*x
    sage: numpy.float32('1.5') * x
    1.50000000000000*x
"""

from cysignals.memory cimport check_allocarray, check_reallocarray, sig_free
from cysignals.signals cimport sig_on, sig_off

from cpython.long cimport PyLong_AsLong
from cpython.float cimport PyFloat_AS_DOUBLE

from sage.structure.parent cimport Parent
from sage.rings.polynomial.polynomial_element cimport Polynomial, _dict_to_list
from sage.rings.real_mpfr cimport RealField_class, RealNumber
from sage.rings.integer cimport Integer, smallInteger
from sage.rings.rational cimport Rational

from sage.structure.element cimport Element
from sage.structure.element cimport parent
from sage.structure.element import coerce_binop
from sage.libs.mpfr cimport *

try:
    from cypari2.gen import Gen as pari_gen
except ImportError:
    pari_gen = ()


cdef class PolynomialRealDense(Polynomial):
    r"""

    TESTS::

        sage: f = RR['x'].random_element()
        sage: from sage.rings.polynomial.polynomial_real_mpfr_dense import PolynomialRealDense
        sage: isinstance(f, PolynomialRealDense)
        True
    """

    cdef Py_ssize_t _degree
    cdef mpfr_t* _coeffs
    cdef RealField_class _base_ring

    def __cinit__(self):
        """
        TESTS::

            sage: from sage.rings.polynomial.polynomial_real_mpfr_dense import PolynomialRealDense
            sage: PolynomialRealDense(RR['x'])
            0
        """
        self._coeffs = NULL

    def __init__(self, Parent parent, x=0, check=None, bint is_gen=False, construct=None):
        """
        EXAMPLES::

            sage: from sage.rings.polynomial.polynomial_real_mpfr_dense import PolynomialRealDense
            sage: PolynomialRealDense(RR['x'], [1, int(2), RR(3), 4/1, pi])             # needs sage.symbolic
            3.14159265358979*x^4 + 4.00000000000000*x^3 + 3.00000000000000*x^2 + 2.00000000000000*x + 1.00000000000000
            sage: PolynomialRealDense(RR['x'], None)
            0

        TESTS:

        Check that errors and interrupts are handled properly (see :issue:`10100`)::

            sage: a = var('a')                                                          # needs sage.symbolic
            sage: PolynomialRealDense(RR['x'], [1,a])                                   # needs sage.symbolic
            Traceback (most recent call last):
            ...
            TypeError: cannot evaluate symbolic expression to a numeric value
            sage: R.<x> = SR[]                                                          # needs sage.symbolic
            sage: (x-a).change_ring(RR)                                                 # needs sage.symbolic
            Traceback (most recent call last):
            ...
            TypeError: cannot evaluate symbolic expression to a numeric value
            sage: sig_on_count()
            0

        Test that we don't clean up uninitialized coefficients (:issue:`9826`)::

            sage: k.<a> = GF(7^3)                                                       # needs sage.rings.finite_rings
            sage: P.<x> = PolynomialRing(k)                                             # needs sage.rings.finite_rings
            sage: (a*x).complex_roots()                                                 # needs sage.rings.finite_rings
            Traceback (most recent call last):
            ...
            TypeError: unable to convert 'a' to a real number

        Check that :issue:`17190` is fixed::

            sage: RR['x']({})
            0
        """
        Polynomial.__init__(self, parent, is_gen=is_gen)
        self._base_ring = parent._base
        cdef Py_ssize_t i, degree
        cdef int prec = self._base_ring._prec
        cdef mpfr_rnd_t rnd = self._base_ring.rnd
        if x is None:
            self._coeffs = <mpfr_t*>check_allocarray(1, sizeof(mpfr_t)) # degree zero
            mpfr_init2(self._coeffs[0], prec)
            mpfr_set_si(self._coeffs[0], 0, rnd)
            self._normalize()
            return
        if is_gen:
            x = [0, 1]
        elif isinstance(x, (int, float, Integer, Rational, RealNumber)):
            x = [x]
        elif isinstance(x, dict):
            x = _dict_to_list(x, self._base_ring.zero())
        elif isinstance(x, pari_gen):
            x = [self._base_ring(w) for w in x.list()]
        elif not isinstance(x, list):
            try:
                x = list(x)
            except TypeError:  # x is not iterable
                x = [self._base_ring(x)]

        sig_on()
        degree = len(x) - 1
        self._degree = -1
        cdef mpfr_t* coeffs
        coeffs = <mpfr_t*>check_allocarray(degree+1, sizeof(mpfr_t))
        try:  # We might get Python exceptions here
            for i from 0 <= i <= degree:
                mpfr_init2(coeffs[i], prec)
                self._degree += 1
                a = x[i]
                if type(a) is RealNumber:
                    mpfr_set(coeffs[i], (<RealNumber>a).value, rnd)
                elif type(a) is int:
                    mpfr_set_si(coeffs[i], PyLong_AsLong(a), rnd)
                elif type(a) is float:
                    mpfr_set_d(coeffs[i], PyFloat_AS_DOUBLE(a), rnd)
                elif type(a) is Integer:
                    mpfr_set_z(coeffs[i], (<Integer>a).value, rnd)
                elif type(a) is Rational:
                    mpfr_set_q(coeffs[i], (<Rational>a).value, rnd)
                else:
                    a = self._base_ring(a)
                    mpfr_set(coeffs[i], (<RealNumber>a).value, rnd)
        finally:
            sig_off()
        self._coeffs = coeffs
        self._normalize()

    def __dealloc__(self):
        cdef Py_ssize_t i
        if self._coeffs != NULL:
            for i from 0 <= i <= self._degree:
                mpfr_clear(self._coeffs[i])
            sig_free(self._coeffs)

    def __reduce__(self):
        """
        EXAMPLES::

            sage: from sage.rings.polynomial.polynomial_real_mpfr_dense import PolynomialRealDense
            sage: f = PolynomialRealDense(RR['x'], [-2, 0, 1])
            sage: loads(dumps(f)) == f
            True
        """
        return make_PolynomialRealDense, (self._parent, self.list())

    cdef _normalize(self):
        """
        Remove all leading 0's.
        """
        cdef Py_ssize_t i
        if self._degree >= 0 and mpfr_zero_p(self._coeffs[self._degree]):
            i = self._degree
            while i >= 0 and mpfr_zero_p(self._coeffs[i]):
                mpfr_clear(self._coeffs[i])
                i -= 1
            self._coeffs = <mpfr_t*>check_reallocarray(self._coeffs, i+1, sizeof(mpfr_t))
            self._degree = i

    cdef get_unsafe(self, Py_ssize_t i):
        """
        Return the `i`-th coefficient of ``self``.

        EXAMPLES::

            sage: from sage.rings.polynomial.polynomial_real_mpfr_dense import PolynomialRealDense
            sage: f = PolynomialRealDense(RR['x'], range(5)); f
            4.00000000000000*x^4 + 3.00000000000000*x^3 + 2.00000000000000*x^2 + x
            sage: f[0]
            0.000000000000000
            sage: f[3]
            3.00000000000000
            sage: f[5]
            0.000000000000000

        Test slices::

            sage: R.<x> = RealField(10)[]
            sage: f = (x+1)^5; f
            x^5 + 5.0*x^4 + 10.*x^3 + 10.*x^2 + 5.0*x + 1.0
            sage: f[:3]
            10.*x^2 + 5.0*x + 1.0
        """
        cdef RealNumber r = <RealNumber>RealNumber.__new__(RealNumber, self._base_ring)
        mpfr_set(r.value, self._coeffs[i], self._base_ring.rnd)
        return r

    cdef PolynomialRealDense _new(self, Py_ssize_t degree):
        cdef Py_ssize_t i
        cdef int prec = self._base_ring._prec
        cdef PolynomialRealDense f = <PolynomialRealDense>PolynomialRealDense.__new__(PolynomialRealDense)
        f._parent = self._parent
        f._base_ring = self._base_ring
        f._degree = degree
        if degree >= 0:
            f._coeffs = <mpfr_t*>check_allocarray(degree+1, sizeof(mpfr_t))
            for i from 0 <= i <= degree:
                mpfr_init2(f._coeffs[i], prec)
        return f

    def degree(self):
        """
        Return the degree of the polynomial.

        EXAMPLES::

            sage: from sage.rings.polynomial.polynomial_real_mpfr_dense import PolynomialRealDense
            sage: f = PolynomialRealDense(RR['x'], [1, 2, 3]); f
            3.00000000000000*x^2 + 2.00000000000000*x + 1.00000000000000
            sage: f.degree()
            2

        TESTS::

            sage: type(f.degree())
            <class 'sage.rings.integer.Integer'>
        """
        return smallInteger(self._degree)

    cpdef Polynomial truncate(self, long n):
        r"""
        Return the polynomial of degree `< n` which is equivalent to ``self``
        modulo `x^n`.

        EXAMPLES::

            sage: from sage.rings.polynomial.polynomial_real_mpfr_dense import PolynomialRealDense
            sage: f = PolynomialRealDense(RealField(10)['x'], [1, 2, 4, 8])
            sage: f.truncate(3)
            4.0*x^2 + 2.0*x + 1.0
            sage: f.truncate(100)
            8.0*x^3 + 4.0*x^2 + 2.0*x + 1.0
            sage: f.truncate(1)
            1.0
            sage: f.truncate(0)
            0
        """
        if n <= 0:
            return self._new(-1)
        if n > self._degree:
            return self
        cdef PolynomialRealDense f = self._new(n-1)
        cdef Py_ssize_t i
        for i from 0 <= i < n:
            mpfr_set(f._coeffs[i], self._coeffs[i], self._base_ring.rnd)
        f._normalize()
        return f

    def truncate_abs(self, RealNumber bound):
        """
        Truncate all high order coefficients below ``bound``.

        EXAMPLES::

            sage: from sage.rings.polynomial.polynomial_real_mpfr_dense import PolynomialRealDense
            sage: f = PolynomialRealDense(RealField(10)['x'], [10^-k for k in range(10)])
            sage: f
            1.0e-9*x^9 + 1.0e-8*x^8 + 1.0e-7*x^7 + 1.0e-6*x^6 + 0.000010*x^5
             + 0.00010*x^4 + 0.0010*x^3 + 0.010*x^2 + 0.10*x + 1.0
            sage: f.truncate_abs(0.5e-6)
            1.0e-6*x^6 + 0.000010*x^5 + 0.00010*x^4 + 0.0010*x^3 + 0.010*x^2 + 0.10*x + 1.0
            sage: f.truncate_abs(10.0)
            0
            sage: f.truncate_abs(1e-100) == f
            True
        """
        cdef Py_ssize_t i
        for i from self._degree >= i >= 0:
            if mpfr_cmpabs(self._coeffs[i], bound.value) >= 0:
                return self.truncate(i+1)
        return self._new(-1)

    cpdef shift(self, Py_ssize_t n):
        r"""
        Return this polynomial multiplied by the power `x^n`. If `n`
        is negative, terms below `x^n` will be discarded. Does not
        change this polynomial.

        EXAMPLES::

            sage: from sage.rings.polynomial.polynomial_real_mpfr_dense import PolynomialRealDense
            sage: f = PolynomialRealDense(RR['x'], [1, 2, 3]); f
            3.00000000000000*x^2 + 2.00000000000000*x + 1.00000000000000
            sage: f.shift(10)
            3.00000000000000*x^12 + 2.00000000000000*x^11 + x^10
            sage: f.shift(-1)
            3.00000000000000*x + 2.00000000000000
            sage: f.shift(-10)
            0

        TESTS::

            sage: f = RR['x'](0)
            sage: f.shift(3).is_zero()
            True
            sage: f.shift(-3).is_zero()
            True
        """
        cdef Py_ssize_t i
        cdef Py_ssize_t nn = 0 if n < 0 else n
        cdef PolynomialRealDense f
        if n == 0 or self._degree < 0:
            return self
        elif self._degree < -n:
            return self._new(-1)
        else:
            f = self._new(self._degree + n)
            for i from 0 <= i < n:
                mpfr_set_ui(f._coeffs[i], 0, self._base_ring.rnd)
            for i from nn <= i <= self._degree + n:
                mpfr_set(f._coeffs[i], self._coeffs[i-n], self._base_ring.rnd)
        return f

    cpdef list list(self, bint copy=True):
        """
        EXAMPLES::

            sage: from sage.rings.polynomial.polynomial_real_mpfr_dense import PolynomialRealDense
            sage: f = PolynomialRealDense(RR['x'], [1, 0, -2]); f
            -2.00000000000000*x^2 + 1.00000000000000
            sage: f.list()
            [1.00000000000000, 0.000000000000000, -2.00000000000000]
        """
        cdef RealNumber r
        cdef Py_ssize_t i
        cdef list L = []
        for i from 0 <= i <= self._degree:
            r = <RealNumber>RealNumber(self._base_ring)
            mpfr_set(r.value, self._coeffs[i], self._base_ring.rnd)
            L.append(r)
        return L

    def __neg__(self):
        """
        EXAMPLES::

            sage: from sage.rings.polynomial.polynomial_real_mpfr_dense import PolynomialRealDense
            sage: f = PolynomialRealDense(RR['x'], [-2,0,1])
            sage: -f
            -x^2 + 2.00000000000000
        """
        cdef Py_ssize_t i
        cdef mpfr_rnd_t rnd = self._base_ring.rnd
        cdef PolynomialRealDense f = self._new(self._degree)
        for i from 0 <= i <= f._degree:
            mpfr_neg(f._coeffs[i], self._coeffs[i], rnd)
        return f

    cpdef _add_(left, _right):
        """
        EXAMPLES::

            sage: from sage.rings.polynomial.polynomial_real_mpfr_dense import PolynomialRealDense
            sage: f = PolynomialRealDense(RR['x'], [-2,0,1]); f
            x^2 - 2.00000000000000
            sage: g = PolynomialRealDense(RR['x'], range(5)); g
            4.00000000000000*x^4 + 3.00000000000000*x^3 + 2.00000000000000*x^2 + x
            sage: f+g
            4.00000000000000*x^4 + 3.00000000000000*x^3 + 3.00000000000000*x^2 + x - 2.00000000000000
            sage: g + f == f + g
            True
            sage: f + (-f)
            0
        """
        cdef Py_ssize_t i
        cdef mpfr_rnd_t rnd = left._base_ring.rnd
        cdef PolynomialRealDense right = _right
        cdef Py_ssize_t min = left._degree if left._degree < right._degree else right._degree
        cdef Py_ssize_t max = left._degree if left._degree > right._degree else right._degree
        cdef PolynomialRealDense f = left._new(max)
        for i from 0 <= i <= min:
            mpfr_add(f._coeffs[i], left._coeffs[i], right._coeffs[i], rnd)
        if left._degree < right._degree:
            for i from min < i <= max:
                mpfr_set(f._coeffs[i], right._coeffs[i], rnd)
        else:
            for i from min < i <= max:
                mpfr_set(f._coeffs[i], left._coeffs[i], rnd)
        f._normalize()
        return f

    cpdef _sub_(left, _right):
        """
        EXAMPLES::

            sage: from sage.rings.polynomial.polynomial_real_mpfr_dense import PolynomialRealDense
            sage: f = PolynomialRealDense(RR['x'], [-3,0,1]); f
            x^2 - 3.00000000000000
            sage: g = PolynomialRealDense(RR['x'], range(4)); g
            3.00000000000000*x^3 + 2.00000000000000*x^2 + x
            sage: f-g
            -3.00000000000000*x^3 - x^2 - x - 3.00000000000000
            sage: (f-g) == -(g-f)
            True
        """
        cdef Py_ssize_t i
        cdef mpfr_rnd_t rnd = left._base_ring.rnd
        cdef PolynomialRealDense right = _right
        cdef Py_ssize_t min = left._degree if left._degree < right._degree else right._degree
        cdef Py_ssize_t max = left._degree if left._degree > right._degree else right._degree
        cdef PolynomialRealDense f = left._new(max)
        for i from 0 <= i <= min:
            mpfr_sub(f._coeffs[i], left._coeffs[i], right._coeffs[i], rnd)
        if left._degree < right._degree:
            for i from min < i <= max:
                mpfr_neg(f._coeffs[i], right._coeffs[i], rnd)
        else:
            for i from min < i <= max:
                mpfr_set(f._coeffs[i], left._coeffs[i], rnd)
        f._normalize()
        return f

    cpdef _lmul_(self, Element c):
        """
        EXAMPLES::

            sage: from sage.rings.polynomial.polynomial_real_mpfr_dense import PolynomialRealDense
            sage: f = PolynomialRealDense(RR['x'], [-5,0,0,1]); f
            x^3 - 5.00000000000000
            sage: 4.0 * f
            4.00000000000000*x^3 - 20.0000000000000
            sage: f * -0.2
            -0.200000000000000*x^3 + 1.00000000000000
        """
        cdef Py_ssize_t i
        cdef RealNumber a = c
        if mpfr_zero_p(a.value):
            return self._new(-1)
        cdef mpfr_rnd_t rnd = self._base_ring.rnd
        cdef PolynomialRealDense f = self._new(self._degree)
        for i from 0 <= i <= self._degree:
            mpfr_mul(f._coeffs[i], self._coeffs[i], a.value, rnd)
        return f

    cpdef _mul_(left, _right):
        """
        Here we use the naive `O(n^2)` algorithm, as asymptotically faster algorithms such
        as Karatsuba can have very inaccurate results due to intermediate rounding errors.

        EXAMPLES::

            sage: from sage.rings.polynomial.polynomial_real_mpfr_dense import PolynomialRealDense
            sage: f = PolynomialRealDense(RR['x'], [1e20, 1])
            sage: g = PolynomialRealDense(RR['x'], [1e30, 1])
            sage: f*g
            x^2 + 1.00000000010000e30*x + 1.00000000000000e50
            sage: f._mul_karatsuba(g,0)
            x^2 + 1.00000000000000e50
            sage: f = PolynomialRealDense(RR['x'], range(5))
            sage: g = PolynomialRealDense(RR['x'], range(3))
            sage: f*g
            8.00000000000000*x^6 + 10.0000000000000*x^5 + 7.00000000000000*x^4 + 4.00000000000000*x^3 + x^2
        """
        cdef Py_ssize_t i, j
        cdef mpfr_rnd_t rnd = left._base_ring.rnd
        cdef PolynomialRealDense right = _right
        cdef PolynomialRealDense f
        cdef mpfr_t tmp
        if left._degree < 0 or right._degree < 0:
            f = left._new(-1)
        else:
            f = left._new(left._degree + right._degree)
        sig_on()
        mpfr_init2(tmp, left._base_ring._prec)
        for i from 0 <= i <= f._degree:
            # Yes, we could make this more efficient by initializing with
            # a multiple of left rather than all zeros...
            mpfr_set_ui(f._coeffs[i], 0, rnd)
        for i from 0 <= i <= left._degree:
            for j from 0 <= j <= right._degree:
                mpfr_mul(tmp, left._coeffs[i], right._coeffs[j], rnd)
                mpfr_add(f._coeffs[i+j], f._coeffs[i+j], tmp, rnd)
        mpfr_clear(tmp)
        sig_off()
        return f

    def _derivative(self, var=None):
        """
        EXAMPLES::

            sage: from sage.rings.polynomial.polynomial_real_mpfr_dense import PolynomialRealDense
            sage: f = PolynomialRealDense(RR['x'], [pi, 0, 2, 1])                       # needs sage.symbolic
            sage: f.derivative()                                                        # needs sage.symbolic
            3.00000000000000*x^2 + 4.00000000000000*x

        TESTS::

            sage: x, y = var('x,y')                                                     # needs sage.symbolic
            sage: f.derivative(x)                                                       # needs sage.symbolic
            3.00000000000000*x^2 + 4.00000000000000*x
            sage: f.derivative(y)                                                       # needs sage.symbolic
            Traceback (most recent call last):
            ...
            ValueError: cannot differentiate with respect to y
        """
        if var is not None and var != self._parent.gen():
            raise ValueError("cannot differentiate with respect to {}".format(var))

        cdef mpfr_rnd_t rnd = self._base_ring.rnd
        cdef PolynomialRealDense f = self._new(self._degree-1)
        for i from 0 <= i < self._degree:
            mpfr_mul_ui(f._coeffs[i], self._coeffs[i+1], i+1, rnd)
        return f

    def integral(self):
        """
        EXAMPLES::

            sage: from sage.rings.polynomial.polynomial_real_mpfr_dense import PolynomialRealDense
            sage: f = PolynomialRealDense(RR['x'], [3, pi, 1])                          # needs sage.symbolic
            sage: f.integral()                                                          # needs sage.symbolic
            0.333333333333333*x^3 + 1.57079632679490*x^2 + 3.00000000000000*x
        """
        cdef mpfr_rnd_t rnd = self._base_ring.rnd
        cdef PolynomialRealDense f = self._new(self._degree+1)
        mpfr_set_ui(f._coeffs[0], 0, rnd)
        for i from 0 <= i <= self._degree:
            mpfr_div_ui(f._coeffs[i+1], self._coeffs[i], i+1, rnd)
        return f

    def reverse(self, degree=None):
        """
        Return reverse of the input polynomial thought as a polynomial of
        degree ``degree``.

        If `f` is a degree-`d` polynomial, its reverse is `x^d f(1/x)`.

        INPUT:

        - ``degree`` -- ``None`` or an integer; if specified, truncate or zero
          pad the list of coefficients to this degree before reversing it

        EXAMPLES::

            sage: f = RR['x']([-3, pi, 0, 1])
            sage: f.reverse()
            -3.00000000000000*x^3 + 3.14159265358979*x^2 + 1.00000000000000
            sage: f.reverse(2)
            -3.00000000000000*x^2 + 3.14159265358979*x
            sage: f.reverse(5)
            -3.00000000000000*x^5 + 3.14159265358979*x^4 + x^2

        TESTS:

        We check that this implementation is compatible with the generic one::

            sage: all(f.reverse(d) == Polynomial.reverse(f, d)                          # needs sage.symbolic
            ....:     for d in [None, 0, 1, 2, 3, 4, 5])
            True
        """
        if degree is None: degree = self._degree

        cdef mpfr_rnd_t rnd = self._base_ring.rnd
        cdef PolynomialRealDense f = self._new(degree)

        cdef int i
        for i in range(1+min(degree, self._degree)):
            mpfr_set(f._coeffs[degree-i], self._coeffs[i], rnd)
        for i in range(1 + self._degree, 1 + degree):
            mpfr_set_si(f._coeffs[degree-i], 0, rnd)
        f._normalize()
        return f

    @coerce_binop
    def quo_rem(self, PolynomialRealDense other):
        """
        Return the quotient with remainder of ``self`` by ``other``.

        EXAMPLES::

            sage: from sage.rings.polynomial.polynomial_real_mpfr_dense import PolynomialRealDense
            sage: f = PolynomialRealDense(RR['x'], [-2, 0, 1])
            sage: g = PolynomialRealDense(RR['x'], [5, 1])
            sage: q, r = f.quo_rem(g)
            sage: q
            x - 5.00000000000000
            sage: r
            23.0000000000000
            sage: q*g + r == f
            True
            sage: fg = f*g
            sage: fg.quo_rem(f)
            (x + 5.00000000000000, 0)
            sage: fg.quo_rem(g)
            (x^2 - 2.00000000000000, 0)

            sage: f = PolynomialRealDense(RR['x'], range(5))
            sage: g = PolynomialRealDense(RR['x'], [pi,3000,4])
            sage: q, r = f.quo_rem(g)
            sage: g*q + r == f
            True

        TESTS:

        Check that :issue:`18467` is fixed::

            sage: S.<x> = RR[]
            sage: z = S.zero()
            sage: z.degree()
            -1
            sage: q, r = z.quo_rem(x)
            sage: q.degree()
            -1
        """
        if other._degree < 0:
            raise ZeroDivisionError("other must be nonzero")
        elif other._degree == 0:
            return self * ~other[0], self._parent.zero()
        elif other._degree > self._degree:
            return self._parent.zero(), self
        cdef mpfr_rnd_t rnd = self._base_ring.rnd
        cdef PolynomialRealDense q, r
        cdef Py_ssize_t i, j
        cdef mpfr_t tmp
        # Make divisor monic for simplicity
        leading = other[other._degree]
        other = other * ~leading
        r = self * ~leading
        q = self._new(self._degree - other._degree)
        # This is the standard division algorithm
        sig_on()
        mpfr_init2(tmp, self._base_ring._prec)
        for i from self._degree >= i >= other._degree:
            mpfr_set(q._coeffs[i-other._degree], r._coeffs[i], rnd)
            for j from 0 <= j < other._degree:
                mpfr_mul(tmp, r._coeffs[i], other._coeffs[j], rnd)
                mpfr_sub(r._coeffs[i-other._degree+j], r._coeffs[i-other._degree+j], tmp, rnd)
            r._degree -= 1
            mpfr_clear(r._coeffs[i])
        mpfr_clear(tmp)
        sig_off()
        r._normalize()
        return q, r * leading

    def __call__(self, *args, **kwds):
        """
        EXAMPLES::

            sage: from sage.rings.polynomial.polynomial_real_mpfr_dense import PolynomialRealDense
            sage: f = PolynomialRealDense(RR['x'], [-2, 0, 1])
            sage: f(10)
            98.0000000000000
            sage: f(CC.0)
            -3.00000000000000
            sage: f(2.0000000000000000000000000000000000000000000)
            2.00000000000000
            sage: f(RealField(10)(2))
            2.0
            sage: f(pi)                                                                 # needs sage.symbolic
            1.00000000000000*pi^2 - 2.00000000000000


            sage: f = PolynomialRealDense(RR['x'], range(5))
            sage: f(1)
            10.0000000000000
            sage: f(-1)
            2.00000000000000
            sage: f(0)
            0.000000000000000
            sage: f = PolynomialRealDense(RR['x'])
            sage: f(12)
            0.000000000000000

        TESTS::

            sage: R.<x> = RR[]  # Issue #17311
            sage: (x^2+1)(x=5)
            26.0000000000000
        """
        if len(args) == 1:
            xx = args[0]
        else:
            return Polynomial.__call__(self, *args, **kwds)

        if not isinstance(xx, RealNumber):
            if self._base_ring.has_coerce_map_from(parent(xx)):
                xx = self._base_ring(xx)
            else:
                return Polynomial.__call__(self, xx)

        cdef Py_ssize_t i
        cdef mpfr_rnd_t rnd = self._base_ring.rnd
        cdef RealNumber x = <RealNumber>xx
        cdef RealNumber res

        if (<RealField_class>x._parent)._prec < self._base_ring._prec:
            res = RealNumber(x._parent)
        else:
            res = RealNumber(self._base_ring)
        # Optimize some very useful and common cases:
        if self._degree < 0:
            mpfr_set_ui(res.value, 0, rnd)
        elif mpfr_zero_p(x.value):
            mpfr_set(res.value, self._coeffs[0], rnd)
        elif mpfr_cmp_ui(x.value, 1) == 0:
            mpfr_set(res.value, self._coeffs[0], rnd)
            for i from 0 < i <= self._degree:
                mpfr_add(res.value, res.value, self._coeffs[i], rnd)
        elif mpfr_cmp_si(x.value, -1) == 0:
            mpfr_set(res.value, self._coeffs[0], rnd)
            for i from 2 <= i <= self._degree by 2:
                mpfr_add(res.value, res.value, self._coeffs[i], rnd)
            for i from 1 <= i <= self._degree by 2:
                mpfr_sub(res.value, res.value, self._coeffs[i], rnd)
        else:
            mpfr_set(res.value, self._coeffs[self._degree], rnd)
            for i from self._degree > i >= 0:
                mpfr_mul(res.value, res.value, x.value, rnd)
                mpfr_add(res.value, res.value, self._coeffs[i], rnd)
        return res

    def change_ring(self, R):
        """
        EXAMPLES::

            sage: from sage.rings.polynomial.polynomial_real_mpfr_dense import PolynomialRealDense
            sage: f = PolynomialRealDense(RR['x'], [-2, 0, 1.5])
            sage: f.change_ring(QQ)
            3/2*x^2 - 2
            sage: f.change_ring(RealField(10))
            1.5*x^2 - 2.0
            sage: f.change_ring(RealField(100))
            1.5000000000000000000000000000*x^2 - 2.0000000000000000000000000000
        """
        cdef Py_ssize_t i
        cdef mpfr_rnd_t rnd = self._base_ring.rnd
        cdef PolynomialRealDense f
        if isinstance(R, RealField_class):
            f = PolynomialRealDense(R[self.variable_name()])
            f = f._new(self._degree)
            for i from 0 <= i <= self._degree:
                mpfr_set(f._coeffs[i], self._coeffs[i], rnd)
            return f
        else:
            return Polynomial.change_ring(self, R)


def make_PolynomialRealDense(parent, data):
    """
    EXAMPLES::

        sage: from sage.rings.polynomial.polynomial_real_mpfr_dense import make_PolynomialRealDense
        sage: make_PolynomialRealDense(RR['x'], [1,2,3])
        3.00000000000000*x^2 + 2.00000000000000*x + 1.00000000000000
    """
    return PolynomialRealDense(parent, data)



"""
Theta Series of Quadratic Forms

AUTHORS:

- Jonathan Hanke: initial code, theta series of degree 1

- Gonzalo Tornaria (2009-02-22): fixes and doctests

- Gonzalo Tornaria (2010-03-23): theta series of degree 2
"""
from copy import deepcopy

from sage.rings.power_series_ring import PowerSeriesRing
from sage.rings.integer_ring import ZZ


def theta_series(self, Max=10, var_str='q', safe_flag=True):
    r"""
    Compute the theta series as a power series in the variable given
    in ``var_str`` (which defaults to ``'q'``), up to the specified precision
    `O(q^{Max})`.

    This uses the PARI/GP function :pari:`qfrep`, wrapped by the
    theta_by_pari() method. This caches the result for future
    computations.

    The ``safe_flag`` allows us to select whether we want a copy of the
    output, or the original output.  It is only meaningful when a
    vector is returned, otherwise a copy is automatically made in
    creating the power series.  By default ``safe_flag`` = ``True``, so we
    return a copy of the cached information.  If this is set to ``False``,
    then the routine is much faster but the return values are
    vulnerable to being corrupted by the user.

    .. TODO::

        Allow the option ``Max='mod_form'`` to give enough coefficients
        to ensure we determine the theta series as a modular form.  This
        is related to the Sturm bound, but we will need to be careful about
        this (particularly for half-integral weights!).

    EXAMPLES::

        sage: Q = DiagonalQuadraticForm(ZZ, [1,3,5,7])
        sage: Q.theta_series()                                                          # needs sage.libs.pari
        1 + 2*q + 2*q^3 + 6*q^4 + 2*q^5 + 4*q^6 + 6*q^7 + 8*q^8 + 14*q^9 + O(q^10)

        sage: Q.theta_series(25)                                                        # needs sage.libs.pari
        1 + 2*q + 2*q^3 + 6*q^4 + 2*q^5 + 4*q^6 + 6*q^7 + 8*q^8 + 14*q^9 + 4*q^10
        + 12*q^11 + 18*q^12 + 12*q^13 + 12*q^14 + 8*q^15 + 34*q^16 + 12*q^17 + 8*q^18
        + 32*q^19 + 10*q^20 + 28*q^21 + 16*q^23 + 44*q^24 + O(q^25)
    """
    # Sanity Check: Max is an integer or an allowed string:
    try:
        M = ZZ(Max)
    except TypeError:
        M = -1

    if Max not in ['mod_form'] and not M >= 0:
        raise TypeError("Max = {Max} is not an integer >= 0 or an allowed string")

    if Max == 'mod_form':
        raise NotImplementedError("we have to figure out the correct number of Fourier coefficients to use")
        # return self.theta_by_pari(sturm_bound(self.level(), self.dim() / ZZ(2)) + 1, var_str, safe_flag)
    else:
        return self.theta_by_pari(M, var_str, safe_flag)


# ---- Compute the theta function by using the PARI/GP routine qfrep  ----

def theta_by_pari(self, Max, var_str='q', safe_flag=True):
    r"""
    Use PARI/GP to compute the theta function as a power series (or
    vector) up to the precision `O(q^{Max})`.  This also caches the result
    for future computations.

    If ``var_str`` = ``''``, then we return a vector `v` where ``v[i]`` counts the
    number of vectors of length `i`.

    The ``safe_flag`` allows us to select whether we want a copy of the
    output, or the original output.  It is only meaningful when a
    vector is returned, otherwise a copy is automatically made in
    creating the power series.  By default ``safe_flag=True``, so we
    return a copy of the cached information.  If this is set to ``False``,
    then the routine is much faster but the return values are
    vulnerable to being corrupted by the user.

    INPUT:

    - ``Max`` -- integer `\geq 0`
    - ``var_str`` -- string

    OUTPUT: a power series or a vector

    EXAMPLES::

        sage: Q = DiagonalQuadraticForm(ZZ, [1,1,1,1])
        sage: Prec = 100
        sage: compute = Q.theta_by_pari(Prec, '')                                       # needs sage.libs.pari
        sage: exact = [1] + [8 * sum([d  for d in divisors(i)  if d % 4 != 0])          # needs sage.libs.pari
        ....:                for i in range(1, Prec)]
        sage: compute == exact                                                          # needs sage.libs.pari
        True
    """
    # Try to use the cached result if it's enough precision
    if hasattr(self, '__theta_vec') and len(self.__theta_vec) >= Max:
        theta_vec = self.__theta_vec[:Max]
    else:
        theta_vec = self.representation_number_list(Max)
        # Cache the theta vector
        self.__theta_vec = theta_vec

    # Return the answer
    if not var_str:
        if safe_flag:
            return deepcopy(theta_vec)         # We must make a copy here to insure the integrity of the cached version!
        return theta_vec
    return PowerSeriesRing(ZZ, var_str)(theta_vec, Max)


# -- Compute the theta function by using an explicit Cholesky decomposition --


########################################################################
# Routines to compute the Fourier expansion of the theta function of Q #
# (to a given precision) via a Cholesky decomposition over RR.         #
#                                                                      #
# The Cholesky code was taken from:                                    #
# ~/Documents/290_Project/C/Ver13.2__3-5-2007/Matrix_mpz/Matrix_mpz.cc #
########################################################################


def theta_by_cholesky(self, q_prec):
    r"""
    Uses the real Cholesky decomposition to compute (the `q`-expansion of) the
    theta function of the quadratic form as a power series in `q` with terms
    correct up to the power `q^{\text{q\_prec}}`. (So its error is `O(q^
    {\text{q\_prec} + 1})`.)

    REFERENCE:

    Cohen's "A Course in Computational Algebraic Number Theory" book, p 102.

    EXAMPLES:

    Check the sum of 4 squares form against Jacobi's formula::

        sage: Q = DiagonalQuadraticForm(ZZ, [1,1,1,1])
        sage: Theta = Q.theta_by_cholesky(10)
        sage: Theta
        1 + 8*q + 24*q^2 + 32*q^3 + 24*q^4 + 48*q^5 + 96*q^6
         + 64*q^7 + 24*q^8 + 104*q^9 + 144*q^10
        sage: Expected = [1] + [8*sum(d for d in divisors(n) if d%4)
        ....:                   for n in range(1, 11)]
        sage: Expected
        [1, 8, 24, 32, 24, 48, 96, 64, 24, 104, 144]
        sage: Theta.list() == Expected
        True

    Check the form `x^2 + 3y^2 + 5z^2 + 7w^2` represents everything except 2 and 22.::

        sage: Q = DiagonalQuadraticForm(ZZ, [1,3,5,7])
        sage: Theta = Q.theta_by_cholesky(50)
        sage: Theta_list = Theta.list()
        sage: [m  for m in range(len(Theta_list))  if Theta_list[m] == 0]
        [2, 22]
    """
    # RAISE AN ERROR -- This routine is deprecated!
    # raise NotImplementedError("This routine is deprecated.  Try theta_series(), which uses theta_by_pari().")

    from sage.arith.misc import integer_ceil as ceil, integer_floor as floor
    from sage.misc.functional import sqrt
    from sage.rings.real_mpfr import RealField

    n = self.dim()
    theta = [0] * (q_prec + 1)
    PS = PowerSeriesRing(ZZ, 'q')

    bit_prec = 53                  # TO DO: Set this precision to reflect the appropriate roundoff
    Cholesky = self.cholesky_decomposition(bit_prec)     # error estimate, to be confident through our desired q-precision.
    Q = Cholesky      # <----  REDUNDANT!!!
    R = RealField(bit_prec)
    half = R(0.5)

    # 1. Initialize
    i = n - 1
    T = [R.zero()] * n
    U = [R.zero()] * n
    T[i] = R(q_prec)
    U[i] = 0
    L = [0] * n
    x = [0] * n

    # 2. Compute bounds
    # Z = sqrt(T[i] / Q[i,i])      # IMPORTANT NOTE: sqrt preserves the precision of the real number it's given... which is not so good... =|
    # L[i] = floor(Z - U[i])       # Note: This is a Sage Integer
    # x[i] = ceil(-Z - U[i]) - 1   # Note: This is a Sage Integer too

    done_flag = False
    from_step4_flag = False
    from_step3_flag = True        # We start by pretending this, since then we get to run through 2 and 3a once. =)

    # Big loop which runs through all vectors
    while not done_flag:

        # Loop through until we get to i=1 (so we defined a vector x)
        while from_step3_flag or from_step4_flag:              # IMPORTANT WARNING:  This replaces a do...while loop, so it may have to be adjusted!

            # Go to directly to step 3 if we're coming from step 4, otherwise perform step 2.
            if from_step4_flag:
                from_step4_flag = False
            else:
                # 2. Compute bounds
                from_step3_flag = False
                Z = sqrt(T[i] / Q[i, i])
                L[i] = floor(Z - U[i])
                x[i] = ceil(-Z - U[i]) - 1

            # 3a. Main loop
            x[i] += 1
            while x[i] > L[i]:
                i += 1
                x[i] += 1

            # 3b. Main loop
            if i > 0:
                from_step3_flag = True
                T[i - 1] = T[i] - Q[i, i] * (x[i] + U[i]) * (x[i] + U[i])
                i += -1
                U[i] = 0
                for j in range(i + 1, n):
                    U[i] += Q[i, j] * x[j]

        # 4. Solution found (This happens when i=0)
        from_step4_flag = True
        Q_val_double = q_prec - T[0] + Q[0, 0] * (x[0] + U[0]) * (x[0] + U[0])
        Q_val = floor(Q_val_double + half)        # Note: This rounds the value up, since the "round" function returns a float, but floor returns integer.

        # OPTIONAL SAFETY CHECK:
        eps = 0.000000001
        if abs(Q_val_double - Q_val) > eps:
            raise RuntimeError("Oh No! We have a problem with the floating point precision... \n"
                               + " Q_val_double = " + str(Q_val_double) + "\n"
                               + " Q_val = " + str(Q_val) + "\n"
                               + " x = " + str(x) + "\n")

        if Q_val <= q_prec:
            theta[Q_val] += 2

        # 5. Check if x = 0, for exit condition. =)
        done_flag = all(x[j] == 0 for j in range(n))

    # Set the value: theta[0] = 1
    theta[0] = 1

    # Return the series, truncated to the desired q-precision
    return PS(theta)


def theta_series_degree_2(Q, prec) -> dict:
    r"""
    Compute the theta series of degree 2 for the quadratic form `Q`.

    INPUT:

    - ``prec`` -- integer

    OUTPUT: dictionary, where:

    - keys are `{\rm GL}_2(\ZZ)`-reduced binary quadratic forms (given as triples of
      coefficients)
    - values are coefficients

    EXAMPLES::

        sage: # needs sage.symbolic
        sage: Q2 = QuadraticForm(ZZ, 4, [1,1,1,1, 1,0,0, 1,0, 1])
        sage: S = Q2.theta_series_degree_2(10)
        sage: S[(0,0,2)]
        24
        sage: S[(1,0,1)]
        144
        sage: S[(1,1,1)]
        192

    AUTHORS:

    - Gonzalo Tornaria (2010-03-23)

    REFERENCE:

    - Raum, Ryan, Skoruppa, Tornaria, 'On Formal Siegel Modular Forms'
      (preprint)
    """
    from sage.arith.misc import integer_floor as floor
    from sage.misc.functional import sqrt
    from sage.misc.timing import cputime
    from sage.misc.verbose import verbose

    if Q.base_ring() != ZZ:
        raise TypeError("the quadratic form must be integral")
    if not Q.is_positive_definite():
        raise ValueError("the quadratic form must be positive definite")
    try:
        X = ZZ(prec - 1)    # maximum discriminant
    except TypeError:
        raise TypeError("prec is not an integer")

    if X < -1:
        raise ValueError("prec must be >= 0")

    if X == -1:
        return {}

    V = ZZ ** Q.dim()
    H = Q.Hessian_matrix()

    t = cputime()
    maxi = (X + 1) // 4
    v_list = (Q.vectors_by_length(maxi))        # assume a>0
    v_list = [[V(c) for c in vs] for vs in v_list]  # coerce vectors into V
    verbose("Computed vectors_by_length", t)

    # Deal with the singular part
    coeffs = {(0, 0, 0): ZZ.one()}
    for i in range(1, maxi + 1):
        coeffs[(0, 0, i)] = ZZ(2) * len(v_list[i])

    # Now deal with the non-singular part
    a_max = int(floor(sqrt(X / 3)))
    for a in range(1, a_max + 1):
        t = cputime()
        c_max = int(floor((a * a + X) / (4 * a)))
        for c in range(a, c_max + 1):
            for v1 in v_list[a]:
                v1_H = v1 * H

                def B_v1(v):
                    return v1_H * v2
                for v2 in v_list[c]:
                    b = abs(B_v1(v2))
                    if b <= a and 4 * a * c - b * b <= X:
                        qf = (a, b, c)
                        count = ZZ(4) if b == 0 else ZZ(2)
                        coeffs[qf] = coeffs.get(qf, ZZ.zero()) + count
        verbose("done a = %d" % a, t)

    return coeffs



# sage.doctest: needs sage.libs.pari
"""
TESTS:

Test consistency between modular forms and modular symbols Eisenstein
series charpolys, which are computed in completely separate ways. ::

    sage: for N in range(25,33):
    ....:    m = ModularForms(N)
    ....:    e = m.eisenstein_subspace()
    ....:    f = e.hecke_polynomial(2)
    ....:    g = ModularSymbols(N).eisenstein_submodule().hecke_polynomial(2)
    ....:    print("{} {}".format(N, f == g))
    25 True
    26 True
    27 True
    28 True
    29 True
    30 True
    31 True
    32 True

Another similar consistency check::

    sage: for N in range(1,26):
    ....:    eps = (N > 2 and DirichletGroup(N,QQ).0) or N
    ....:    m = ModularForms(eps)
    ....:    e = m.eisenstein_subspace()
    ....:    f = e.hecke_polynomial(2)
    ....:    g = ModularSymbols(eps).eisenstein_submodule().hecke_polynomial(2)
    ....:    print("{} {}".format(N, f == g))
    1 True
    2 True
    3 True
    4 True
    5 True
    6 True
    7 True
    8 True
    9 True
    10 True
    11 True
    12 True
    13 True
    14 True
    15 True
    16 True
    17 True
    18 True
    19 True
    20 True
    21 True
    22 True
    23 True
    24 True
    25 True

We check that bug :issue:`8541` (traced to a linear algebra problem) is fixed::

    sage: f = CuspForms(DirichletGroup(5).0,5).0
    sage: f[15]
    30*zeta4 - 210
"""



r"""
Interface to Mathics

Mathics is an open source interpreter for the Wolfram Language.
From the introduction of its reference manual:

.. NOTE::

    Mathics — to be pronounced like “Mathematics” without the
    “emat” — is a general-purpose computer algebra system (CAS).
    It is meant to be a free, light-weight alternative to
    Mathematica®. It is free both as in “free beer” and as in
    “freedom”. There are various online mirrors running
    Mathics but it is also possible to run Mathics locally.
    A list of mirrors can be found at the Mathics homepage,
    http://mathics.github.io.

    The programming language of Mathics is meant to resemble
    Wolfram’s famous Mathematica® as much as possible. However,
    Mathics is in no way affiliated or supported by Wolfram.
    Mathics will probably never have the power to compete with
    Mathematica® in industrial applications; yet, it might be
    an interesting alternative for educational purposes.

The Mathics interface will only work if the optional Sage package Mathics
is installed. The interface lets you send certain Sage objects to Mathics,
run Mathics functions, import certain Mathics expressions to Sage,
or any combination of the above.

To send a Sage object ``sobj`` to Mathics, call ``mathics(sobj)``.
This exports the Sage object to Mathics and returns a new Sage object
wrapping the Mathics expression/variable, so that you can use the
Mathics variable from within Sage. You can then call Mathics
functions on the new object; for example::

    sage: from sage.interfaces.mathics import mathics
    sage: mobj = mathics(x^2-1); mobj       # optional - mathics
    -1 + x ^ 2
    sage: mobj.Factor()                     # optional - mathics
    (-1 + x) (1 + x)

In the above example the factorization is done using Mathics's
``Factor[]`` function.

To see Mathics's output you can simply print the Mathics wrapper
object. However if you want to import Mathics's output back to Sage,
call the Mathics wrapper object's ``sage()`` method. This method returns
a native Sage object::

    sage: # optional - mathics
    sage: mobj = mathics(x^2-1)
    sage: mobj2 = mobj.Factor(); mobj2
    (-1 + x) (1 + x)
    sage: mobj2.parent()
    Mathics
    sage: sobj = mobj2.sage(); sobj
    (x + 1)*(x - 1)
    sage: sobj.parent()
    Symbolic Ring


If you want to run a Mathics function and don't already have the input
in the form of a Sage object, then it might be simpler to input a string to
``mathics(expr)``. This string will be evaluated as if you had typed it
into Mathics::

    sage: mathics('Factor[x^2-1]')          # optional - mathics
    (-1 + x) (1 + x)
    sage: mathics('Range[3]')               # optional - mathics
    {1, 2, 3}

If you want work with the internal Mathics expression, then you can call
``mathics.eval(expr)``, which returns an instance of
:class:`mathics.core.expression.Expression`. If you want the result to
be a string formatted like Mathics's InputForm, call ``repr(mobj)`` on
the wrapper object ``mobj``. If you want a string formatted in Sage style,
call ``mobj._sage_repr()``::

    sage: mathics.eval('x^2 - 1')           # optional - mathics
    '-1 + x ^ 2'
    sage: repr(mathics('Range[3]'))         # optional - mathics
    '{1, 2, 3}'
    sage: mathics('Range[3]')._sage_repr()  # optional - mathics
    '[1, 2, 3]'

Finally, if you just want to use a Mathics command line from within
Sage, the function ``mathics_console()`` dumps you into an interactive
command-line Mathics session.

Tutorial
--------

We follow some of the tutorial from
http://library.wolfram.com/conferences/devconf99/withoff/Basic1.html/.


Syntax
~~~~~~

Now make 1 and add it to itself. The result is a Mathics
object.

::

    sage: m = mathics
    sage: a = m(1) + m(1); a                # optional - mathics
    2
    sage: a.parent()                        # optional - mathics
    Mathics
    sage: m('1+1')                          # optional - mathics
    2
    sage: m(3)**m(50)                       # optional - mathics
    717897987691852588770249

The following is equivalent to ``Plus[2, 3]`` in
Mathics::

    sage: m = mathics
    sage: m(2).Plus(m(3))                   # optional - mathics
    5

We can also compute `7(2+3)`.

::

    sage: m(7).Times(m(2).Plus(m(3)))       # optional - mathics
    35
    sage: m('7(2+3)')                       # optional - mathics
    35

Some typical input
~~~~~~~~~~~~~~~~~~

We solve an equation and a system of two equations::

    sage: # optional - mathics
    sage: eqn = mathics('3x + 5 == 14')
    sage: eqn
    5 + 3 x == 14
    sage: eqn.Solve('x')
    {{x -> 3}}
    sage: sys = mathics('{x^2 - 3y == 3, 2x - y == 1}')
    sage: print(sys)
    {x ^ 2 - 3 y == 3, 2 x - y == 1}
    sage: sys.Solve('{x, y}')
    {{x -> 0, y -> -1}, {x -> 6, y -> 11}}

Assignments and definitions
~~~~~~~~~~~~~~~~~~~~~~~~~~~

If you assign the mathics `5` to a variable `c`
in Sage, this does not affect the `c` in Mathics.

::

    sage: c = m(5)                          # optional - mathics
    sage: print(m('b + c x'))               # optional - mathics
                 b + c x
    sage: print(m('b') + c*m('x'))          # optional - mathics
             b + 5 x

The Sage interfaces changes Sage lists into Mathics lists::

    sage: m = mathics
    sage: eq1 = m('x^2 - 3y == 3')          # optional - mathics
    sage: eq2 = m('2x - y == 1')            # optional - mathics
    sage: v = m([eq1, eq2]); v              # optional - mathics
    {x ^ 2 - 3 y == 3, 2 x - y == 1}
    sage: v.Solve(['x', 'y'])               # optional - mathics
    {{x -> 0, y -> -1}, {x -> 6, y -> 11}}

Function definitions
~~~~~~~~~~~~~~~~~~~~

Define mathics functions by simply sending the definition to
the interpreter.

::

    sage: m = mathics
    sage: _ = mathics('f[p_] = p^2');       # optional - mathics
    sage: m('f[9]')                         # optional - mathics
    81

Numerical Calculations
~~~~~~~~~~~~~~~~~~~~~~

We find the `x` such that `e^x - 3x = 0`.

::

    sage: eqn = mathics('Exp[x] - 3x == 0') # optional - mathics
    sage: eqn.FindRoot(['x', 2])            # optional - mathics
    {x -> 1.51213}

Note that this agrees with what the PARI interpreter gp produces::

    sage: gp('solve(x=1,2,exp(x)-3*x)')
    1.5121345516578424738967396780720387046

Next we find the minimum of a polynomial using the two different
ways of accessing Mathics::

    sage: mathics('FindMinimum[x^3 - 6x^2 + 11x - 5, {x,3}]')  # not tested (since not supported, so far)
    {0.6150998205402516, {x -> 2.5773502699629733}}
    sage: f = mathics('x^3 - 6x^2 + 11x - 5')                  # optional - mathics
    sage: f.FindMinimum(['x', 3])                              # not tested (since not supported, so far)
    {0.6150998205402516, {x -> 2.5773502699629733}}

Polynomial and Integer Factorization
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

We factor a polynomial of degree 200 over the integers.

::

    sage: R.<x> = PolynomialRing(ZZ)
    sage: f = (x**100+17*x+5)*(x**100-5*x+20)
    sage: f
    x^200 + 12*x^101 + 25*x^100 - 85*x^2 + 315*x + 100
    sage: g = mathics(str(f))                # optional - mathics
    sage: print(g)                           # optional - mathics
    100 + 315 x - 85 x ^ 2 + 25 x ^ 100 + 12 x ^ 101 + x ^ 200
    sage: g                                  # optional - mathics
    100 + 315 x - 85 x ^ 2 + 25 x ^ 100 + 12 x ^ 101 + x ^ 200
    sage: print(g.Factor())                  # optional - mathics
    (5 + 17 x + x ^ 100) (20 - 5 x + x ^ 100)

We can also factor a multivariate polynomial::

    sage: f = mathics('x^6 + (-y - 2)*x^5 + (y^3 + 2*y)*x^4 - y^4*x^3')  # optional - mathics
    sage: print(f.Factor())                  # optional - mathics
    x ^ 3 (x - y) (-2 x + x ^ 2 + y ^ 3)

We factor an integer::

    sage: # optional - mathics
    sage: n = mathics(2434500)
    sage: n.FactorInteger()
    {{2, 2}, {3, 2}, {5, 3}, {541, 1}}
    sage: n = mathics(2434500)
    sage: F = n.FactorInteger(); F
    {{2, 2}, {3, 2}, {5, 3}, {541, 1}}
    sage: F[1]
    {2, 2}
    sage: F[4]
    {541, 1}


Long Input
----------

The Mathics interface reads in even very long input (using
files) in a robust manner.

::

    sage: t = '"%s"'%10^10000   # ten thousand character string.
    sage: a = mathics(t)        # optional - mathics
    sage: a = mathics.eval(t)   # optional - mathics

Loading and saving
------------------

Mathics has an excellent ``InputForm`` function,
which makes saving and loading Mathics objects possible. The
first examples test saving and loading to strings.

::

    sage: # optional - mathics
    sage: x = mathics(pi/2)
    sage: print(x)
    Pi / 2
    sage: loads(dumps(x)) == x
    True
    sage: n = x.N(50)
    sage: print(n)
                  1.5707963267948966192313216916397514420985846996876
    sage: loads(dumps(n)) == n
    True

Complicated translations
------------------------

The ``mobj.sage()`` method tries to convert a Mathics object to a Sage
object. In many cases, it will just work. In particular, it should be able to
convert expressions entirely consisting of:

- numbers, i.e. integers, floats, complex numbers;
- functions and named constants also present in Sage, where:

    - Sage knows how to translate the function or constant's name from
      Mathics's, or
    - the Sage name for the function or constant is trivially related to
      Mathics's;

- symbolic variables whose names don't pathologically overlap with
  objects already defined in Sage.

This method will not work when Mathics's output includes:

- strings;
- functions unknown to Sage;
- Mathics functions with different parameters/parameter order to
  the Sage equivalent.

If you want to convert more complicated Mathics expressions, you can
instead call ``mobj._sage_()`` and supply a translation dictionary::

    sage: x = var('x')
    sage: m = mathics('NewFn[x]')                 # optional - mathics
    sage: m._sage_(locals={'NewFn': sin, 'x':x})  # optional - mathics
    sin(x)

For more details, see the documentation for ``._sage_()``.


OTHER Examples::

    sage: def math_bessel_K(nu, x):
    ....:     return mathics(nu).BesselK(x).N(20)
    sage: math_bessel_K(2,I)                      # optional - mathics
    -2.5928861754911969782 + 0.18048997206696202663 I

::

    sage: slist = [[1, 2], 3., 4 + I]
    sage: mlist = mathics(slist); mlist         # optional - mathics
    {{1, 2}, 3., 4 + I}
    sage: slist2 = list(mlist); slist2          # optional - mathics
    [{1, 2}, 3., 4 + I]
    sage: slist2[0]                             # optional - mathics
    {1, 2}
    sage: slist2[0].parent()                    # optional - mathics
    Mathics
    sage: slist3 = mlist.sage(); slist3         # optional - mathics
    [[1, 2], 3.00000000000000, 4.00000000000000 + 1.00000000000000*I]

::

    sage: mathics('10.^80')         # optional - mathics
    1.×10^80
    sage: mathics('10.^80').sage()  # optional - mathics
    1.00000000000000e80

AUTHORS:

- Sebastian Oehms (2021): first version from a copy of the Mathematica interface (see :issue:`31778`).


Thanks to Rocky Bernstein and Juan Mauricio Matera for their support. For further acknowledgments see `this list <https://github.com/mathics/Mathics/blob/master/AUTHORS.txt>`__.

TESTS:

Check that numerical approximations via Mathics's `N[]` function work
correctly (:issue:`18888`, :issue:`28907`)::

    sage: # optional - mathics
    sage: mathics('Pi/2').N(10)
    1.570796327
    sage: mathics('Pi').N(10)
    3.141592654
    sage: mathics('Pi').N(50)
    3.1415926535897932384626433832795028841971693993751
    sage: str(mathics('Pi*x^2-1/2').N())
    '-0.5 + 3.14159 x ^ 2.'

Check that Mathics's `E` exponential symbol is correctly backtranslated
as Sage's `e` (:issue:`29833`)::

    sage: (e^x)._mathics_().sage()  # optional -- mathics
    e^x
    sage: exp(x)._mathics_().sage() # optional -- mathics
    e^x
"""

##############################################################################
#       Copyright (C) 2021 Sebastian Oehms <seb.oehms@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
##############################################################################

import os

from sage.misc.cachefunc import cached_method
from sage.interfaces.interface import Interface, InterfaceElement, InterfaceFunction, InterfaceFunctionElement
from sage.interfaces.tab_completion import ExtraTabCompletion
from sage.misc.instancedoc import instancedoc
from sage.structure.richcmp import rich_to_bool


def _mathics_sympysage_symbol(self):
    r"""
    Convert a Sympy symbol ``self`` to a corresponding element
    in Sage's symbolic ring.

    This function replaces ``_sympysage_symbol`` to
    take care of the special names used in Mathics.
    It is set to the method ``_sage_`` of the Sympy class
    :class:`sympy.core.symbol.Sympol`.

    EXAMPLES::

        sage: # optional - mathics
        sage: from sage.interfaces.mathics import _mathics_sympysage_symbol
        sage: mt = mathics('t')
        sage: st = mt.to_sympy(); st
        _Mathics_User_Global`t
        sage: _mathics_sympysage_symbol(st)
        t
        sage: bool(_ == st._sage_())
        True
        sage: type(st._sage_())
        <class 'sage.symbolic.expression.Expression'>
    """
    from sage.symbolic.ring import SR
    try:
        name = self.name
        if name.startswith('_Mathics_User_'):
            name = name.split('`')[1]
            if name == mathics._true_symbol():
                return True
            if name == mathics._false_symbol():
                return False
        return SR.var(name)
    except ValueError:
        # sympy sometimes returns dummy variables
        # with name = 'None', str rep = '_None'
        # in particular in inverse Laplace and inverse Mellin transforms
        return SR.var(str(self))


class Mathics(Interface):
    r"""
    Interface to the Mathics interpreter.

    Implemented according to the Mathematica interface but avoiding Pexpect
    functionality.

    EXAMPLES::

        sage: # optional - mathics
        sage: t = mathics('Tan[I + 0.5]')
        sage: t.parent()
        Mathics
        sage: ts = t.sage()
        sage: ts.parent()
        Complex Field with 53 bits of precision
        sage: t == mathics(ts)
        True
        sage: mtan = mathics.Tan
        sage: mt = mtan(I+1/2)
        sage: mt == t
        True
        sage: u = mathics(I+1/2)
        sage: u.Tan() == mt
        True


    More examples can be found in the module header.
    """
    def __init__(self,
                 maxread=None,
                 logfile=None,
                 init_list_length=1024,
                 seed=None):
        r"""
        Python constructor.

        EXAMPLES::

            sage: mathics._mathics_init_ == mathics._mathematica_init_
            True
        """

        Interface.__init__(self, name='mathics')
        self._seed = seed
        self._initialized = False  # done lazily
        self._session = None
        os.environ['MATHICS_CHARACTER_ENCODING'] = 'ASCII'  # see :issue:`37395`

    def _lazy_init(self):
        r"""
        Initialize the Mathics interpreter.

        Implemented according to R interface.

        EXAMPLES::

            sage: mathics._lazy_init()   # optional - mathics
        """
        if not self._initialized:
            self._initialized = True
            self._start()

    def _start(self):
        """
        Start up the Mathics interpreter and sets the initial prompt and options.

        This is called the first time the Mathics interface is actually used.

        EXAMPLES::

            sage: mathics._start()            # optional - mathics
            sage: type(mathics._session)      # optional - mathics
            <class 'mathics.session.MathicsSession'>
        """
        if not self._session:
            from mathics.session import MathicsSession
            from mathics.core.load_builtin import import_and_load_builtins
            import_and_load_builtins()
            self._session = MathicsSession()
            from sage.interfaces.sympy import sympy_init
            sympy_init()
            from sympy import Symbol
            Symbol._sage_ = _mathics_sympysage_symbol

    def _read_in_file_command(self, filename):
        r"""
        EXAMPLES::

            sage: from sage.misc.temporary_file import tmp_filename
            sage: fn = tmp_filename()
            sage: mathics('40!>>%s' %fn)                     # optional - mathics
            815915283247897734345611269596115894272000000000
            sage: mathics(mathics._read_in_file_command(fn)) # optional - mathics
            815915283247897734345611269596115894272000000000
            sage: os.system('rm %s' %fn)                     # optional - mathics
            0
        """
        return '<<"%s"' % filename

    def _install_hints(self):
        """
        Hints for installing mathics on your computer.

        EXAMPLES::

            sage: len(mathics._install_hints())  # optional - mathics
            101
        """
        return """
In order to use the Mathics interface you need to have the
optional Sage package Mathics installed.
"""

    def _eval(self, code):
        """
        Evaluates a command inside the Mathics interpreter and returns the output
        as a Mathics result.

        EXAMPLES::

            sage: mathics._eval('1+1').last_eval  # optional - mathics
            <Integer: 2>
        """
        self._lazy_init()
        S = self._session
        expr = S.evaluate(code)
        from mathics.core.evaluation import Evaluation
        ev = Evaluation(S.definitions)
        return ev.evaluate(expr)

    def eval(self, code, *args, **kwds):
        """
        Evaluates a command inside the Mathics interpreter and returns the output
        in printable form.

        EXAMPLES::

            sage: mathics.eval('1+1')  # optional - mathics
            '2'
        """
        res = self._eval(code)
        if res.result == 'Null':
            if len(res.out) == 1:
                return str(res.out[0])
        return res.result

    def set(self, var, value):
        """
        Set the variable var to the given value.

        EXAMPLES::

            sage: mathics.set('u', '2*x +E')         # optional - mathics
            sage: bool(mathics('u').sage() == 2*x+e) # optional - mathics
            True
        """
        cmd = f'{var}={value};'
        _ = self.eval(cmd)

    def get(self, var):
        """
        Get the value of the variable var.

        EXAMPLES::

            sage: mathics.set('u', '2*x +E')        # optional - mathics
            sage: mathics.get('u')                  # optional - mathics
            '2 x + E'
        """
        return self.eval(var)

    def _function_call_string(self, function, args, kwds):
        """
        Return the string used to make function calls.

        EXAMPLES::

            sage: mathics._function_call_string('Sin', ['x'], [])
            'Sin[x]'
        """
        return "{}[{}]".format(function, ",".join(args))

    def _left_list_delim(self):
        r"""
        EXAMPLES::

            sage: mathics._left_list_delim()
            '{'
        """
        return "{"

    def _right_list_delim(self):
        r"""
        EXAMPLES::

            sage: mathics._right_list_delim()
            '}'
        """
        return "}"

    def _left_func_delim(self):
        r"""
        EXAMPLES::

            sage: mathics._left_func_delim()
            '['
        """
        return "["

    def _right_func_delim(self):
        r"""
        EXAMPLES::

            sage: mathics._right_func_delim()
            ']'
        """
        return "]"

    # #########################################
    # System -- change directory, etc.
    # #########################################
    def chdir(self, dir):
        """
        Change Mathics's current working directory.

        EXAMPLES::

            sage: mathics.chdir('/')          # optional - mathics
            sage: mathics('Directory[]')      # optional - mathics
            /
        """
        self.eval('SetDirectory["%s"]' % dir)

    def _true_symbol(self):
        r"""
        EXAMPLES::

            sage: mathics._true_symbol()
            'True'
        """
        return 'True'

    def _false_symbol(self):
        r"""
        EXAMPLES::

            sage: mathics._false_symbol()
            'False'
        """
        return 'False'

    def _equality_symbol(self):
        r"""
        EXAMPLES::

            sage: mathics._equality_symbol()
            '=='
        """
        return '=='

    def _assign_symbol(self):
        r"""
        EXAMPLES::

            sage: mathics._assign_symbol()
            ':='
        """
        return ':='

    def _exponent_symbol(self):
        r"""
        Return the symbol used to denote the exponent of a number in
        Mathics.

        EXAMPLES::

            sage: mathics._exponent_symbol()
            '*^'

        ::

            sage: bignum = mathics('10.^80')       # optional - mathics
            sage: repr(bignum)                     # optional - mathics
            '1.×10^80'
            sage: repr(bignum).replace(mathics._exponent_symbol(), 'e').strip() # optional - mathics
            '1.×10^80'
        """
        return '*^'

    def _object_class(self):
        r"""
        Return the element class of this parent.
        This is used in the interface class.

        EXAMPLES::

            sage: mathics._object_class()
            <class 'sage.interfaces.mathics.MathicsElement'>
        """
        return MathicsElement

    def console(self):
        r"""
        Spawn a new Mathics command-line session.

        EXAMPLES::

            sage: mathics.console()  # not tested

            Mathics 2.1.1.dev0
            on CPython 3.9.2 (default, Mar 19 2021, 22:23:28)
            using SymPy 1.7, mpmath 1.2.1, numpy 1.19.5, cython 0.29.21

            Copyright (C) 2011-2021 The Mathics Team.
            This program comes with ABSOLUTELY NO WARRANTY.
            This is free software, and you are welcome to redistribute it
            under certain conditions.
            See the documentation for the full license.

            Quit by evaluating Quit[] or by pressing CONTROL-D.

            In[1]:= Sin[0.5]
            Out[1]= 0.479426

            Goodbye!

            sage:
        """
        mathics_console()

    def help(self, cmd, long=False):
        r"""
        Return the Mathics documentation of the given command.

        EXAMPLES::

            sage: mathics.help('Sin')                   # optional - mathics
            'sine function\n'

            sage: print(_)                              # optional - mathics
            sine function
            <BLANKLINE>

            sage: print(mathics.help('Sin', long=True)) # optional - mathics
            sine function
            <BLANKLINE>
            Attributes[Sin] = {Listable, NumericFunction, Protected}
            <BLANKLINE>

            sage: print(mathics.Factorial.__doc__)  # optional - mathics
            factorial
            <BLANKLINE>

            sage: u = mathics('Pi')                 # optional - mathics
            sage: print(u.Cos.__doc__)              # optional - mathics
            cosine function
            <BLANKLINE>
        """
        if long:
            return self.eval('Information[%s]' % cmd)
        return self.eval('? %s' % cmd)

    def __getattr__(self, attrname):
        r"""
        EXAMPLES::

            sage: msin = mathics.Sin           # optional - mathics
            sage: msin(0.2)                    # optional - mathics
            0.19866933079506123
            sage: _ == sin(0.2)                # optional - mathics
            True
        """
        if attrname[:1] == "_":
            raise AttributeError
        return InterfaceFunction(self, attrname)


@instancedoc
class MathicsElement(ExtraTabCompletion, InterfaceElement):
    r"""
    Element class of the Mathics interface.

    Its instances are usually constructed via the instance call of its parent.
    It wrapes the Mathics library for this object. In a session Mathics methods
    can be obtained using tab completion.

    EXAMPLES::

        sage: # optional - mathics
        sage: me=mathics(e); me
        E
        sage: type(me)
        <class 'sage.interfaces.mathics.MathicsElement'>
        sage: P = me.parent(); P
        Mathics
        sage: type(P)
        <class 'sage.interfaces.mathics.Mathics'>

    Access to the Mathics expression objects::

        sage: # optional - mathics
        sage: res = me._mathics_result
        sage: type(res)
        <class 'mathics.core.evaluation.Result'>
        sage: expr = res.last_eval; expr
        <Symbol: System`E>
        sage: type(expr)
        <class 'mathics.core.symbols.Symbol'>

    Applying Mathics methods::

        sage: # optional - mathics
        sage: me.to_sympy()
        E
        sage: me.get_name()
        'System`E'
        sage: me.is_inexact()
        False

    Conversion to Sage::

        sage: bool(me.sage() == e)             # optional - mathics
        True
    """

    def _tab_completion(self):
        r"""
        Return a list of all methods of this object.

        .. NOTE::

           Currently returns all methods of :class:`mathics.expression.Expression`.

        EXAMPLES::

            sage: a = mathics(5*x)             # optional - mathics
            sage: t = a._tab_completion()      # optional - mathics
            sage: len(t) > 100                 # optional - mathics
            True
        """
        return dir(self._mathics_result.last_eval)

    def __getitem__(self, n):
        r"""
        EXAMPLES::

            sage: l = mathics('{1, x, .15}')  # optional - mathics
            sage: l[0]                        # optional - mathics
            List
            sage: for i in l: print(i)        # optional - mathics
            1
            x
            0.15
        """
        return self.parent().new(f'{self._name}[[{n}]]')

    def __getattr__(self, attrname):
        r"""
        EXAMPLES::

            sage: # optional - mathics
            sage: a = mathics(5*x)
            sage: res = a._mathics_result
            sage: str(a) == res.result
            True
            sage: t = mathics._eval('5*x')
            sage: t.last_eval  == res.last_eval
            True
        """
        P = self._check_valid()
        if attrname == '_mathics_result':
            self._mathics_result = P._eval(self.name())
            return self._mathics_result
        if attrname[:1] == "_":
            raise AttributeError
        else:
            expr = self._mathics_result.last_eval
            if hasattr(expr, attrname):
                return expr.__getattribute__(attrname)
        return InterfaceFunctionElement(self, attrname)

    def __float__(self, precision=16):
        r"""
        EXAMPLES::

            sage: float(mathics('Pi')) == float(pi)  # optional - mathics
            True
        """
        P = self.parent()
        return float(P._eval(f'N[{self.name()},{precision}]').last_eval.to_mpmath())

    def _reduce(self):
        r"""
        EXAMPLES::

            sage: slist = [[1, 2], 3., 4 + I]
            sage: mlist = mathics(slist)    # optional - mathics
            sage: mlist._reduce()           # optional - mathics
            '{{1, 2}, 3., 4 + I}'
        """
        return str(self)

    def __reduce__(self):
        r"""
        EXAMPLES::

            sage: mpol = mathics('x + y*z')     # optional - mathics
            sage: loads(dumps(mpol)) == mpol    # optional - mathics
            True
        """
        return reduce_load, (self._reduce(), )

    def _latex_(self):
        r"""
        EXAMPLES::

            sage: Q = mathics('Sin[x Cos[y]]/Sqrt[1-x^2]')   # optional - mathics
            sage: latex(Q)                                   # optional - mathics
            \frac{\text{Sin}\left[x \text{Cos}\left[y\right]\right]}{\sqrt{1-x^2}}
        """
        z = str(self.parent()('TeXForm[%s]' % self.name()))
        i = z.find('=')
        return z[i + 1:]

    def _repr_(self):
        r"""
        EXAMPLES::

            sage: Q = mathics('Sin[x Cos[y]]/Sqrt[1-x^2]')   # optional - mathics
            sage: repr(Q)                                    # optional - mathics
            'Sin[x Cos[y]] / Sqrt[1 - x ^ 2]'
        """
        return self._mathics_result.result

    def _sage_(self, locals={}):
        r"""
        Attempt to return a Sage version of this object.

        This method works successfully when Mathics returns a result
        or list of results that consist only of:

        - numbers, i.e. integers, floats, complex numbers;
        - functions and named constants also present in Sage, where:
            - Sage knows how to translate the function or constant's name
              from Mathics's naming scheme, or
            - you provide a translation dictionary `locals`, or
            - the Sage name for the function or constant is simply the
              Mathics name in lower case;

        - symbolic variables whose names do not pathologically overlap with
          objects already defined in Sage.

        This method will not work when Mathics's output includes:

        - strings;
        - functions unknown to Sage;
        - Mathics functions with different parameters/parameter order to
          the Sage equivalent. In this case, define a function to do the
          parameter conversion, and pass it in via the locals dictionary.

        EXAMPLES:

        Mathics lists of numbers/constants become Sage lists of
        numbers/constants::

            sage: # optional - mathics
            sage: m = mathics('{{1., 4}, Pi, 3.2e100, I}')
            sage: s = m.sage(); s
            [[1.00000000000000, 4], pi, 3.20000000000000*e100, 1.00000000000000*I]
            sage: s[1].n()
            3.14159265358979
            sage: s[3]^2
            -1.00000000000000

        ::

            sage: m = mathics('x^2 + 5*y')      # optional - mathics
            sage: m.sage()                      # optional - mathics
            x^2 + 5*y

        ::

            sage: m = mathics('Sin[Sqrt[1-x^2]] * (1 - Cos[1/x])^2')  # optional - mathics
            sage: m.sage()                          # optional - mathics
            (cos(1/x) - 1)^2*sin(sqrt(-x^2 + 1))

        ::

            sage: m = mathics('NewFn[x]')                 # optional - mathics
            sage: m._sage_(locals={'NewFn': sin, 'x':x})  # optional - mathics
            sin(x)

        ::

            sage: var('bla')                        # optional - mathics
            bla
            sage: m = mathics('bla^2')              # optional - mathics
            sage: bla^2 - m.sage()                  # optional - mathics
            0

        ::

            sage: # optional - mathics
            sage: m = mathics('bla^2')
            sage: mb = m.sage()
            sage: var('bla')
            bla
            sage: bla^2 - mb
            0
        """
        if locals:
            # if locals are given we use `_sage_repr`
            # surely this only covers simple cases
            from sage.misc.sage_eval import sage_eval
            return sage_eval(self._sage_repr(), locals=locals)

        self._check_valid()
        if self.is_inexact():
            m = self.to_mpmath()
            if self is not m and m is not None:
                from sage.libs.mpmath.utils import mpmath_to_sage
                return mpmath_to_sage(m, self.get_precision())
        s = self.to_sympy()
        if self is not s and s is not None:
            if hasattr(s, '_sage_'):
                try:
                    return s._sage_()
                except NotImplementedError:  # see :issue:`33584`
                    pass
        p = self.to_python()
        if self is not p and p is not None:
            def conv(i):
                return self.parent()(i).sage()
            if isinstance(p, list):
                return [conv(i) for i in p]
            if isinstance(p, tuple):
                return tuple([conv(i) for i in p])
            if type(p) is dict:
                return {conv(k): conv(v) for k, v in p.items()}
            return p
        return s

    def __len__(self):
        """
        Return the object's length, evaluated by mathics.

        EXAMPLES::

            sage: len(mathics([1,1.,2]))    # optional - mathics
            3
        """
        return int(self.Length())

    @cached_method
    def _is_graphics(self):
        """
        Test whether the mathics expression is graphics.

        OUTPUT: boolean

        EXAMPLES::

            sage: P = mathics('Plot[Sin[x],{x,-2Pi,4Pi}]')   # optional - mathics
            sage: P._is_graphics()                           # optional - mathics
            True
        """
        return str(self).startswith('-Graphics-')

    def save_image(self, filename, ImageSize=600):
        r"""
        Save a mathics graphics.

        INPUT:

        - ``filename`` -- string; the filename to save as. The
          extension determines the image file format

        - ``ImageSize`` -- integer; the size of the resulting image

        EXAMPLES::

            sage: P = mathics('Plot[Sin[x],{x,-2Pi,4Pi}]')   # optional - mathics
            sage: filename = tmp_filename()                  # optional - mathics
            sage: P.save_image(filename, ImageSize=800)      # optional - mathics
        """
        P = self._check_valid()
        if not self._is_graphics():
            raise ValueError('mathics expression is not graphics')
        filename = os.path.abspath(filename)
        s = f'Export["{filename}", {self.name()}, ImageSize->{ImageSize}]'
        P.eval(s)

    def _rich_repr_(self, display_manager, **kwds):
        """
        Rich Output Magic Method.

        See :mod:`sage.repl.rich_output` for details.

        EXAMPLES::

            sage: from sage.repl.rich_output import get_display_manager
            sage: dm = get_display_manager()
            sage: P = mathics('Plot[Sin[x],{x,-2Pi,4Pi}]')       # optional - mathics

        The following test requires a working X display on Linux so that the
        Mathematica frontend can do the rendering (:issue:`23112`)::

            sage: P._rich_repr_(dm)                              # optional - mathics
            OutputImageSvg container
        """
        if self._is_graphics():
            OutputImageSvg = display_manager.types.OutputImageSvg
            if display_manager.preferences.graphics == 'disable':
                return
            if OutputImageSvg in display_manager.supported_output():
                return display_manager.graphics_from_save(
                    self.save_image, kwds, '.svg', OutputImageSvg)
        else:
            OutputLatex = display_manager.types.OutputLatex
            dmp = display_manager.preferences.text
            if dmp is None or dmp == 'plain':
                return
            if dmp == 'latex' and OutputLatex in display_manager.supported_output():
                return OutputLatex(self._latex_())

    def show(self, ImageSize=600):
        r"""
        Show a mathics expression immediately.

        This method attempts to display the graphics immediately,
        without waiting for the currently running code (if any) to
        return to the command line. Be careful, calling it from within
        a loop will potentially launch a large number of external
        viewer programs.

        INPUT:

        - ``ImageSize`` -- integer; the size of the resulting image

        OUTPUT:

        This method does not return anything. Use :meth:`save` if you
        want to save the figure as an image.

        EXAMPLES::

            sage: Q = mathics('Sin[x Cos[y]]/Sqrt[1-x^2]')   # optional - mathics
            sage: show(Q)                                    # optional - mathics
            Sin[x Cos[y]] / Sqrt[1 - x ^ 2]

            sage: P = mathics('Plot[Sin[x],{x,-2Pi,4Pi}]')   # optional - mathics
            sage: show(P)                                    # optional - mathics
            sage: P.show(ImageSize=800)                      # optional - mathics
        """
        from sage.repl.rich_output import get_display_manager
        dm = get_display_manager()
        dm.display_immediately(self, ImageSize=ImageSize)

    def _richcmp_(self, other, op):
        r"""
        EXAMPLES::

            sage: # optional - mathics
            sage: mobj1 = mathics([x^2-1, 2])
            sage: mobj2 = mathics('{x^2-1, 2}')
            sage: mobj3 = mathics('5*x + y')
            sage: mobj1 == mobj2
            True
            sage: mobj1 < mobj2
            False
            sage: mobj1 == mobj3
            False
        """
        P = self.parent()
        if str(P(f"{self.name()} < {other.name()}")) == P._true_symbol():
            return rich_to_bool(op, -1)
        if str(P(f"{self.name()} > {other.name()}")) == P._true_symbol():
            return rich_to_bool(op, 1)
        if str(P(f"{self.name()} == {other.name()}")) == P._true_symbol():
            return rich_to_bool(op, 0)
        return NotImplemented

    def __bool__(self):
        """
        Return whether this Mathics element is not identical to ``False``.

        EXAMPLES::

            sage: bool(mathics(True))   # optional - mathics
            True
            sage: bool(mathics(False))  # optional - mathics
            False

        In Mathics, `0` cannot be used to express falsity::

            sage: bool(mathics(0))     # optional - mathics
            True
        """
        P = self._check_valid()
        cmd = f'{self._name}==={P._false_symbol()}'
        return not str(P(cmd)) == P._true_symbol()

    def n(self, *args, **kwargs):
        r"""
        Numerical approximation by converting to Sage object first.

        Convert the object into a Sage object and return its numerical
        approximation. See documentation of the function
        :func:`sage.misc.functional.n` for details.

        EXAMPLES::

            sage: mathics('Pi').n(10)    # optional -- mathics
            3.1
            sage: mathics('Pi').n()      # optional -- mathics
            3.14159265358979
            sage: mathics('Pi').n(digits=10)   # optional -- mathics
            3.141592654
        """
        return self._sage_().n(*args, **kwargs)


# An instance
mathics = Mathics()


def reduce_load(X):
    """
    Used in unpickling a Mathics element.

    This function is just the ``__call__`` method of the interface instance.

    EXAMPLES::

        sage: sage.interfaces.mathics.reduce_load('Denominator[a / b]')  # optional -- mathics
        b
    """

    return mathics(X)


def mathics_console():
    r"""
    Spawn a new Mathics command-line session.

    EXAMPLES::

        sage: mathics_console()  # not tested

        Mathics 2.1.1.dev0
        on CPython 3.9.2 (default, Mar 19 2021, 22:23:28)
        using SymPy 1.7, mpmath 1.2.1, numpy 1.19.5, cython 0.29.21

        Copyright (C) 2011-2021 The Mathics Team.
        This program comes with ABSOLUTELY NO WARRANTY.
        This is free software, and you are welcome to redistribute it
        under certain conditions.
        See the documentation for the full license.

        Quit by evaluating Quit[] or by pressing CONTROL-D.

        In[1]:= Sin[0.5]
        Out[1]= 0.479426

        Goodbye!
    """
    from sage.repl.rich_output.display_manager import get_display_manager
    if not get_display_manager().is_in_terminal():
        raise RuntimeError('Can use the console only in the terminal. Try %%mathics magics instead.')
    from mathics import main
    main.main()



# sage.doctest: needs sage.libs.flint
r"""
`q`-expansion of `j`-invariant
"""
from sage.modular.modform.eis_series import eisenstein_series_qexp
from sage.modular.modform.vm_basis import delta_qexp
from sage.rings.rational_field import QQ


def j_invariant_qexp(prec=10, K=QQ):
    r"""
    Return the `q`-expansion of the `j`-invariant to
    precision ``prec`` in the field `K`.

    .. SEEALSO::

        If you want to evaluate (numerically) the `j`-invariant at
        certain points, see the special function :func:`elliptic_j`.

    .. WARNING::

        Stupid algorithm -- we divide by Delta, which is slow.

    EXAMPLES::

        sage: j_invariant_qexp(4)
        q^-1 + 744 + 196884*q + 21493760*q^2 + 864299970*q^3 + O(q^4)
        sage: j_invariant_qexp(2)
        q^-1 + 744 + 196884*q + O(q^2)
        sage: j_invariant_qexp(100, GF(2))
        q^-1 + q^7 + q^15 + q^31 + q^47 + q^55 + q^71 + q^87 + O(q^100)
    """
    if prec <= -1:
        raise ValueError("the prec must be nonnegative.")
    prec += 2
    g6 = -504*eisenstein_series_qexp(6, prec, K=QQ)
    Delta = delta_qexp(prec).change_ring(QQ)
    j = (g6*g6) * (~Delta) + 1728
    if K != QQ:
        return j.change_ring(K)
    return j


# NOTE: this needs to be sped up.  The pari code src/basemath/trans3.c is
# faster.



"""
Reduction Theory
"""
from copy import deepcopy
from sage.matrix.constructor import matrix
from sage.misc.lazy_import import lazy_import
from sage.misc.mrange import mrange
from sage.modules.free_module_element import vector
from sage.rings.integer_ring import ZZ
lazy_import("sage.functions.all", "floor")


def reduced_binary_form1(self):
    r"""
    Reduce the form `ax^2 + bxy+cy^2` to satisfy the reduced condition `|b| \le
    a \le c`, with `b \ge 0` if `a = c`. This reduction occurs within the
    proper class, so all transformations are taken to have determinant 1.

    EXAMPLES::

        sage: QuadraticForm(ZZ, 2, [5,5,2]).reduced_binary_form1()                      # needs sage.symbolic
        (
        Quadratic form in 2 variables over Integer Ring with coefficients:
        [ 2 -1 ]
        [ * 2 ]                                                            ,
        <BLANKLINE>
        [ 0 -1]
        [ 1  1]
        )

    TESTS::

        sage: QuadraticForm(ZZ, 2, [4,-7,6]).reduced_binary_form1()[0]                  # needs sage.symbolic
        Quadratic form in 2 variables over Integer Ring with coefficients:
        [ 3 -1 ]
        [ * 4 ]

        sage: QuadraticForm(ZZ, 3, [1,2,3,4,5,6]).reduced_binary_form1()                # needs sage.symbolic
        Traceback (most recent call last):
        ...
        TypeError: only available for binary forms
    """
    if self.dim() != 2:
        raise TypeError("only available for binary forms")

    R = self.base_ring()
    interior_reduced_flag = False
    Q = deepcopy(self)
    M = matrix(R, 2, 2, [1, 0, 0, 1])

    while not interior_reduced_flag:
        interior_reduced_flag = True

        # Arrange for a <= c
        if Q[0, 0] > Q[1, 1]:
            M_new = matrix(R, 2, 2, [0, -1, 1, 0])
            Q = Q(M_new)
            M = M * M_new
            interior_reduced_flag = False

        # Arrange for |b| <= a
        if abs(Q[0, 1]) > Q[0, 0]:
            r = R(floor(round(Q[0, 1] / (2 * Q[0, 0]))))
            M_new = matrix(R, 2, 2, [1, -r, 0, 1])
            Q = Q(M_new)
            M = M * M_new
            interior_reduced_flag = False

    return Q, M


def reduced_ternary_form__Dickson(self):
    """
    Find the unique reduced ternary form according to the conditions
    of Dickson's "Studies in the Theory of Numbers", pp164-171.

    EXAMPLES::

        sage: Q = DiagonalQuadraticForm(ZZ, [1, 1, 1])
        sage: Q.reduced_ternary_form__Dickson()
        Traceback (most recent call last):
        ...
        NotImplementedError
    """
    raise NotImplementedError


def reduced_binary_form(self):
    """
    Find a form which is reduced in the sense that no further binary
    form reductions can be done to reduce the original form.

    EXAMPLES::

        sage: QuadraticForm(ZZ, 2, [5,5,2]).reduced_binary_form()                       # needs sage.symbolic
        (
        Quadratic form in 2 variables over Integer Ring with coefficients:
        [ 2 -1 ]
        [ * 2 ]                                                            ,
        <BLANKLINE>
        [ 0 -1]
        [ 1  1]
        )
    """
    R = self.base_ring()
    n = self.dim()
    interior_reduced_flag = False
    Q = deepcopy(self)
    M = matrix(R, n, n, 1)

    while not interior_reduced_flag:
        interior_reduced_flag = True

        # Arrange for (weakly) increasing diagonal entries
        for i in range(n):
            for j in range(i + 1, n):
                if Q[i, i] > Q[j, j]:
                    M_new = matrix(R, n, n, 1)
                    M_new[i, j] = -1
                    M_new[j, i] = 1
                    M_new[i, i] = 0
                    M_new[j, j] = 1

                    Q = Q(M_new)
                    M = M * M_new
                    interior_reduced_flag = False

                # Arrange for |b| <= a
                if abs(Q[i, j]) > Q[i, i]:
                    r = R(floor(round(Q[i, j] / (2 * Q[i, i]))))

                    M_new = matrix(R, n, n, 1)
                    M_new[i, j] = -r

                    Q = Q(M_new)
                    M = M * M_new
                    interior_reduced_flag = False

    return Q, M


def minkowski_reduction(self):
    r"""
    Find a Minkowski-reduced form equivalent to the given one.

    This means that

    .. MATH::

            Q(v_k) \leq Q(s_1\cdot v_1 + ... + s_n\cdot v_n)

    for all `s_i` where `\gcd(s_k, ... s_n) = 1`.

    .. NOTE::

        When `Q` has dim `\leq 4` we can take all `s_i` in `\{1, 0, -1\}`.

    REFERENCES:

    - Schulze-Pillot's paper on "An algorithm for computing genera
      of ternary and quaternary quadratic forms", p138.
    - Donaldson's 1979 paper "Minkowski Reduction of Integral Matrices", p203.

    EXAMPLES::

        sage: Q = QuadraticForm(ZZ, 4, [30, 17, 11, 12, 29, 25, 62, 64, 25, 110])
        sage: Q
        Quadratic form in 4 variables over Integer Ring with coefficients:
        [ 30 17 11 12 ]
        [ * 29 25 62 ]
        [ * * 64 25 ]
        [ * * * 110 ]
        sage: Q.minkowski_reduction()
        (
        Quadratic form in 4 variables over Integer Ring with coefficients:
        [ 30 17 11 -5 ]
        [ * 29 25 4 ]
        [ * * 64 0 ]
        [ * * * 77 ]                                                       ,
        <BLANKLINE>
        [ 1  0  0  0]
        [ 0  1  0 -1]
        [ 0  0  1  0]
        [ 0  0  0  1]
        )

    ::

        sage: Q = QuadraticForm(ZZ,4,[1, -2, 0, 0, 2, 0, 0, 2, 0, 2])
        sage: Q
        Quadratic form in 4 variables over Integer Ring with coefficients:
        [ 1 -2 0 0 ]
        [ * 2 0 0 ]
        [ * * 2 0 ]
        [ * * * 2 ]
        sage: Q.minkowski_reduction()
        (
        Quadratic form in 4 variables over Integer Ring with coefficients:
        [ 1 0 0 0 ]
        [ * 1 0 0 ]
        [ * * 2 0 ]
        [ * * * 2 ]                                                        ,
        <BLANKLINE>
        [1 1 0 0]
        [0 1 0 0]
        [0 0 1 0]
        [0 0 0 1]
        )

    TESTS::

        sage: Q = QuadraticForm(ZZ,5,[2,2,0,0,0,2,2,0,0,2,2,0,2,2,2])
        sage: Q.Gram_matrix()
        [2 1 0 0 0]
        [1 2 1 0 0]
        [0 1 2 1 0]
        [0 0 1 2 1]
        [0 0 0 1 2]
        sage: Q.minkowski_reduction()
        Traceback (most recent call last):
        ...
        NotImplementedError: this algorithm is only for dimensions less than 5

        sage: Q = QuadraticForm(ZZ,2,[4,-11,6])
        sage: Q.minkowski_reduction()
        Traceback (most recent call last):
        ...
        TypeError: Minkowski reduction only works for positive definite forms
    """
    from sage.quadratic_forms.quadratic_form import QuadraticForm
    from sage.quadratic_forms.quadratic_form import matrix
    if not self.is_positive_definite():
        raise TypeError("Minkowski reduction only works for positive definite forms")
    if self.dim() > 4:
        raise NotImplementedError("this algorithm is only for dimensions less than 5")

    R = self.base_ring()
    n = self.dim()
    Q = deepcopy(self)
    M = matrix(R, n, n, 1)

    # Begin the reduction
    done_flag = False
    while not done_flag:

        # Loop through possible shorted vectors until
        done_flag = True
        for j in range(n - 1, -1, -1):
            for a_first in mrange([3 for i in range(j)]):
                y = [x - 1 for x in a_first] + [1] + [0] * (n - 1 - j)
                e_j = [0] * n
                e_j[j] = 1

                # Reduce if a shorter vector is found
                if Q(y) < Q(e_j):

                    # Create the transformation matrix
                    M_new = matrix(R, n, n, 1)
                    for k in range(n):
                        M_new[k, j] = y[k]

                    # Perform the reduction and restart the loop
                    Q = QuadraticForm(M_new.transpose() * Q.matrix() * M_new)
                    M = M * M_new
                    done_flag = False

                if not done_flag:
                    break

            if not done_flag:
                break

    # Return the results
    return Q, M


def minkowski_reduction_for_4vars__SP(self):
    r"""
    Find a Minkowski-reduced form equivalent to the given one.
    This means that

    .. MATH::

        Q(v_k) \leq Q(s_1\cdot v_1 + ... + s_n\cdot v_n)

    for all `s_i` where GCD(`s_k, ... s_n`) = 1.

    .. NOTE::

        When `Q` has dim `\leq 4`, we can take all `s_i` in `\{1, 0, -1\}`.

    REFERENCES:

    - Schulze-Pillot's paper on "An algorithm for computing genera
      of ternary and quaternary quadratic forms", p138.
    - Donaldson's 1979 paper "Minkowski Reduction of Integral Matrices", p203.

    EXAMPLES::

        sage: Q = QuadraticForm(ZZ, 4, [30,17,11,12,29,25,62,64,25,110]); Q
        Quadratic form in 4 variables over Integer Ring with coefficients:
        [ 30 17 11 12 ]
        [ * 29 25 62 ]
        [ * * 64 25 ]
        [ * * * 110 ]
        sage: Q.minkowski_reduction_for_4vars__SP()
        (
        Quadratic form in 4 variables over Integer Ring with coefficients:
        [ 29 -17 25 4 ]
        [ * 30 -11 5 ]
        [ * * 64 0 ]
        [ * * * 77 ]                                                       ,
        <BLANKLINE>
        [ 0  1  0  0]
        [ 1  0  0 -1]
        [ 0  0  1  0]
        [ 0  0  0  1]
        )

    TESTS::

        sage: Q = QuadraticForm(ZZ, 2, [3,4,5])
        sage: Q.minkowski_reduction_for_4vars__SP()
        Traceback (most recent call last):
        ...
        TypeError: the given quadratic form has 2 != 4 variables
    """
    R = self.base_ring()
    n = self.dim()
    Q = deepcopy(self)
    M = matrix(R, n, n, 1)

    # Only allow 4-variable forms
    if n != 4:
        raise TypeError(f"the given quadratic form has {n} != 4 variables")

    # Step 1: Begin the reduction
    done_flag = False
    while not done_flag:

        # Loop through possible shorter vectors
        done_flag = True
        for j in range(n - 1, -1, -1):
            for a_first in mrange([2 for i in range(j)]):
                y = [x - 1 for x in a_first] + [1] + [0] * (n - 1 - j)
                e_j = [0] * n
                e_j[j] = 1

                # Reduce if a shorter vector is found
                if Q(y) < Q(e_j):

                    # Further n=4 computations
                    B_y_vec = Q.matrix() * vector(ZZ, y)
                    # SP's B = our self.matrix()/2
                    # SP's A = coeff matrix of his B
                    # Here we compute the double of both and compare.
                    B_sum = sum([abs(B_y_vec[i]) for i in range(4) if i != j])
                    A_sum = sum([abs(Q[i, j]) for i in range(4) if i != j])
                    B_max = max(abs(B_y_vec[i]) for i in range(4) if i != j)
                    A_max = max(abs(Q[i, j]) for i in range(4) if i != j)

                    if B_sum < A_sum or (B_sum == A_sum and B_max < A_max):

                        # Create the transformation matrix
                        M_new = matrix(R, n, n, 1)
                        for k in range(n):
                            M_new[k, j] = y[k]

                        # Perform the reduction and restart the loop
                        Q = Q(M_new)
                        M = M * M_new
                        done_flag = False

                if not done_flag:
                    break

            if not done_flag:
                break

    # Step 2: Order A by certain criteria
    for i in range(4):
        for j in range(i + 1, 4):

            # Condition (a)
            if Q[i, i] > Q[j, j]:
                Q.swap_variables(i, j, in_place=True)
                M_new = matrix(R, n, n)
                M_new[i, j] = -1
                M_new[j, i] = 1
                for r in range(4):
                    if r == i or r == j:
                        M_new[r, r] = 0
                    else:
                        M_new[r, r] = 1
                M = M * M_new

            elif Q[i, i] == Q[j, j]:
                i_sum = sum([abs(Q[i, k]) for k in range(4) if k != i])
                j_sum = sum([abs(Q[j, k]) for k in range(4) if k != j])

                # Condition (b)
                if i_sum > j_sum:
                    Q.swap_variables(i, j, in_place=True)
                    M_new = matrix(R, n, n)
                    M_new[i, j] = -1
                    M_new[j, i] = 1
                    for r in range(4):
                        if r == i or r == j:
                            M_new[r, r] = 0
                        else:
                            M_new[r, r] = 1
                    M = M * M_new

                elif i_sum == j_sum:
                    for k in [2, 1, 0]:  # TO DO: These steps are a little redundant...
                        Q1 = Q.matrix()

                        c_flag = all(abs(Q1[i, l]) == abs(Q1[j, l])
                                     for l in range(k + 1, 4))

                        # Condition (c)
                        if c_flag and abs(Q1[i, k]) > abs(Q1[j, k]):
                            Q.swap_variables(i, j, in_place=True)
                            M_new = matrix(R, n, n)
                            M_new[i, j] = -1
                            M_new[j, i] = 1
                            for r in range(4):
                                if r == i or r == j:
                                    M_new[r, r] = 0
                                else:
                                    M_new[r, r] = 1
                            M = M * M_new

    # Step 3: Order the signs
    for i in range(4):
        if Q[i, 3] < 0:
            Q.multiply_variable(-1, i, in_place=True)
            M_new = matrix(R, n, n)
            for r in range(4):
                if r == i:
                    M_new[r, r] = -1
                else:
                    M_new[r, r] = 1
            M = M * M_new

    for i in range(4):
        j = 3
        while Q[i, j] == 0:
            j += -1
        if Q[i, j] < 0:
            Q.multiply_variable(-1, i, in_place=True)
            M_new = matrix(R, n, n)
            for r in range(4):
                if r == i:
                    M_new[r, r] = -1
                else:
                    M_new[r, r] = 1
            M = M * M_new

    if Q[1, 2] < 0:
        # Test a row 1 sign change
        if (Q[1, 3] <= 0 and (Q[1, 3] < 0
                              or Q[1, 2] < 0
                              or (Q[1, 2] == 0 and Q[1, 1] < 0))):
            Q.multiply_variable(-1, i, in_place=True)
            M_new = matrix(R, n, n)
            for r in range(4):
                if r == i:
                    M_new[r, r] = -1
                else:
                    M_new[r, r] = 1
            M = M * M_new

        elif (Q[2, 3] <= 0 and ((Q[2, 3] < 0)
                                or Q[2, 2] < 0
                                or (Q[2, 2] == 0 and Q[2, 1] < 0))):
            Q.multiply_variable(-1, i, in_place=True)
            M_new = matrix(R, n, n)
            for r in range(4):
                if r == i:
                    M_new[r, r] = -1
                else:
                    M_new[r, r] = 1
            M = M * M_new

    # Return the results
    return Q, M



# distutils: libraries = flint
# distutils: depends = flint/hypgeom.h

################################################################################
# This file is auto-generated by the script
#   SAGE_ROOT/tools/flint-autogen/flint_autogen.py.
# From the commit 3e2c3a3e091106a25ca9c6fba28e02f2cbcd654a
# Do not modify by hand! Fix and rerun the script instead.
################################################################################

from libc.stdio cimport FILE
from sage.libs.gmp.types cimport *
from sage.libs.mpfr.types cimport *
from sage.libs.flint.types cimport *

cdef extern from "flint_wrap.h":
    void hypgeom_init(hypgeom_t hyp) noexcept
    void hypgeom_clear(hypgeom_t hyp) noexcept
    slong hypgeom_estimate_terms(const mag_t z, int r, slong d) noexcept
    slong hypgeom_bound(mag_t error, int r, slong C, slong D, slong K, const mag_t TK, const mag_t z, slong prec) noexcept
    void hypgeom_precompute(hypgeom_t hyp) noexcept
    void arb_hypgeom_sum(arb_t P, arb_t Q, const hypgeom_t hyp, slong n, slong prec) noexcept
    void arb_hypgeom_infsum(arb_t P, arb_t Q, hypgeom_t hyp, slong tol, slong prec) noexcept
